// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Tools/BinnedHistogram.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/UnstableFinalState.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"

#include "HepMC/GenParticle.h"
#include "HepMC/GenVertex.h"


namespace Rivet {


  class LHCB_2013_I1218996 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructor
    LHCB_2013_I1218996()
      : Analysis("LHCB_2013_I1218996")
    {    }

    //@}


  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      /// Initialise and register projections
      addProjection(UnstableFinalState(), "UFS");

      /// Book histograms
      _h_pdg411_Dplus_pT_y.addHistogram(  2.0, 2.5, bookHistogram1D(3, 1, 1));
      _h_pdg411_Dplus_pT_y.addHistogram(  2.5, 3.0, bookHistogram1D(3, 1, 2));
      _h_pdg411_Dplus_pT_y.addHistogram(  3.0, 3.5, bookHistogram1D(3, 1, 3));
      _h_pdg411_Dplus_pT_y.addHistogram(  3.5, 4.0, bookHistogram1D(3, 1, 4));
      _h_pdg411_Dplus_pT_y.addHistogram(  4.0, 4.5, bookHistogram1D(3, 1, 5));

      _h_pdg421_Dzero_pT_y.addHistogram(  2.0, 2.5, bookHistogram1D(2, 1, 1));
      _h_pdg421_Dzero_pT_y.addHistogram(  2.5, 3.0, bookHistogram1D(2, 1, 2));
      _h_pdg421_Dzero_pT_y.addHistogram(  3.0, 3.5, bookHistogram1D(2, 1, 3));
      _h_pdg421_Dzero_pT_y.addHistogram(  3.5, 4.0, bookHistogram1D(2, 1, 4));
      _h_pdg421_Dzero_pT_y.addHistogram(  4.0, 4.5, bookHistogram1D(2, 1, 5));

      _h_pdg431_Dsplus_pT_y.addHistogram(  2.0, 2.5, bookHistogram1D(5, 1, 1));
      _h_pdg431_Dsplus_pT_y.addHistogram(  2.5, 3.0, bookHistogram1D(5, 1, 2));
      _h_pdg431_Dsplus_pT_y.addHistogram(  3.0, 3.5, bookHistogram1D(5, 1, 3));
      _h_pdg431_Dsplus_pT_y.addHistogram(  3.5, 4.0, bookHistogram1D(5, 1, 4));
      _h_pdg431_Dsplus_pT_y.addHistogram(  4.0, 4.5, bookHistogram1D(5, 1, 5));

      _h_pdg413_Dstarplus_pT_y.addHistogram(  2.0, 2.5, bookHistogram1D(4, 1, 1));
      _h_pdg413_Dstarplus_pT_y.addHistogram(  2.5, 3.0, bookHistogram1D(4, 1, 2));
      _h_pdg413_Dstarplus_pT_y.addHistogram(  3.0, 3.5, bookHistogram1D(4, 1, 3));
      _h_pdg413_Dstarplus_pT_y.addHistogram(  3.5, 4.0, bookHistogram1D(4, 1, 4));
      _h_pdg413_Dstarplus_pT_y.addHistogram(  4.0, 4.5, bookHistogram1D(4, 1, 5));

      _h_pdg4122_Lambdac_pT = bookHistogram1D(1, 1, 1);

    }


    /// Perform the per-event analysis
    void analyze(const Event &event) {
      const double weight = event.weight();
      const UnstableFinalState &ufs = applyProjection<UnstableFinalState> (event, "UFS");

      /// Do the event by event analysis here
      foreach( const Particle &p, ufs.particles() )
      {
        const PdgId id = abs(p.pdgId());

        /// Only interested in charm hadrons
        if( !(PID::isHadron(id) && PID::hasCharm(id)) ) 
          continue;

        /// @note The particle momentum is assumed to be in the proton-proton
        ///   CM frame.
        double y  = p.momentum().rapidity();
        double pT = p.momentum().perp();

        /// Fiducial acceptance of the measurements
        if( pT > 8.0*GeV || y < 2.0 || y > 4.5 )
          continue;

        /// @note Contributions of charm hadrons from the decays of b-hadrons
        ///   and other particles with comparably large mean lifetimes have
        ///   been removed in the measurement.
        /// This is implemented here by counting only charm hadrons that do not
        ///   have an ancestor that contains a b quark.
        if( hasBottomAncestor(p) )
          continue;

        /// @todo Do i need to worry about the possibility that D0 mixing
        /// is implemented as D0 -> D0bar decays that might result in counting
        /// mixed mesons twice?

        switch( id )
        {
          case 411 :
            _h_pdg411_Dplus_pT_y.fill(y, pT/GeV, weight);
            break;

          case 421 :
            _h_pdg421_Dzero_pT_y.fill(y, pT/GeV, weight);
            break;

          case 431 :
            _h_pdg431_Dsplus_pT_y.fill(y, pT/GeV, weight);
            break;

          case 413 :
            _h_pdg413_Dstarplus_pT_y.fill(y, pT/GeV, weight);
            break;

          case 4122 :
            _h_pdg4122_Lambdac_pT->fill(pT/GeV, weight);
            break;
        }
      }
    }


    /// Search ancestry for a b-hadron.
    ///
    /// Adapted from Rivet::Particle::hasAncestor
    bool hasBottomAncestor(const Particle &p) const
    {
      bool result = false;

      HepMC::GenVertex *prodVtx = p.genParticle().production_vertex();

      if(prodVtx)
      {
        foreach ( const HepMC::GenParticle *ancestor, particles(prodVtx, HepMC::ancestors) )
        {
          if( PID::hasBottom(ancestor->pdg_id()) )
          {
            result = true;
            break;
          }
        }
      }

      return result;
    }



    /// Normalise histograms etc., after the run
    void finalize() 
    {
      double scale_factor = (crossSection() / microbarn) / sumOfWeights();

      /// Avoid the implicit division by the bin width in the
      ///   BinnedHistogram::scale method.
      foreach( AIDA::IHistogram1D* hist, _h_pdg411_Dplus_pT_y.getHistograms() )
      {
        hist->scale(scale_factor);
      }

      foreach( AIDA::IHistogram1D* hist, _h_pdg421_Dzero_pT_y.getHistograms() )
      {
        hist->scale(scale_factor);
      }

      foreach( AIDA::IHistogram1D* hist, _h_pdg431_Dsplus_pT_y.getHistograms() )
      {
        hist->scale(scale_factor);
      }

      foreach( AIDA::IHistogram1D* hist, _h_pdg413_Dstarplus_pT_y.getHistograms() )
      {
        hist->scale(scale_factor);
      }

      _h_pdg4122_Lambdac_pT->scale(scale_factor);
    }

    //@}


  private:

    /// @name Histograms
    //@{
    BinnedHistogram<double> _h_pdg411_Dplus_pT_y;
    BinnedHistogram<double> _h_pdg421_Dzero_pT_y;
    BinnedHistogram<double> _h_pdg431_Dsplus_pT_y;
    BinnedHistogram<double> _h_pdg413_Dstarplus_pT_y;
    AIDA::IHistogram1D *_h_pdg4122_Lambdac_pT;
    //@}


  };



  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2013_I1218996);

}
